/*

Qubio.cfg contents : 
	version 0 : List of Hardware Configurations
	version 1 : adds CQUBIOSettings - default new session settings.
	version 2 : adds lpt1 Names  
	version 3 : gain tables 
	version 4 : digital output names
	version 6 : 16 digital output names and separate "[x]Auto Reset" per channel.
	version 7 : removes lpt1 names
*/
#include "stdafx.h"
#include <direct.h>

const char * CONFIGFILENAME = "QUBIO.CFG";		
const char * CONFIGFILETEMP = "QUBIO.NEW";		
const char * CONFIGFILEBACK = "QUBIO.BK";		

char * g_apchDigMode[5]={"Disabled","Auto","Manual","Protocol","Input"};
char * g_ConfigLoc=NULL;


//=============================================================================
CGainTable :: CGainTable() { 
	m_cTableName="Amp Gain";
	m_iEntries=5;
	m_iCurrent=0;
	m_bNew=true;
	for( int iMult=0; iMult<GAIN_MAXENTRIES; ++iMult ) { 
		m_adSignal[iMult]=0.5*iMult;
		m_adMultiplier[iMult]=1.0;
		m_adMin[iMult]=0.5*iMult-0.25;
		m_adMax[iMult]=0.5*iMult+0.25;
		}
	}	

//-----------------------------------------------------------------------------		
void CGainTable :: SetRanges() { 
	if( m_iEntries<2 ) 
		return;
	int i,j;
	double dMid;
	for( i=0; i<m_iEntries; ++i ) { 
		m_adMin[i]=-1.0E8;							// Init range to be quite large
		m_adMax[i]= 1.0E8;
		for( j=0; j<m_iEntries; ++j ) {
			if( j!=i ) { 
				dMid=0.5*(m_adSignal[j]+m_adSignal[i]);
				if( m_adSignal[j] < m_adSignal[i] ) { 
					if( dMid>m_adMin[i] ) 
						m_adMin[i]=dMid;	
					}
				else { 
					if( dMid<m_adMax[i] ) 
						m_adMax[i]=dMid;
					}
				}
			}
		}
	}

//-----------------------------------------------------------------------------		
double CGainTable :: Multiplier( double dSignal ) { 
	// Warning : do not assume table is ordered.
	//-- if prior result no longer applies, search for new result.
	if( m_iCurrent<0 || m_iCurrent>=m_iEntries || dSignal<m_adMin[m_iCurrent] || dSignal>m_adMax[m_iCurrent] ) 
		for( m_iCurrent=m_iEntries-1; m_iCurrent>0; --m_iCurrent )		// default to 0.
			if( dSignal>=m_adMin[m_iCurrent] && dSignal<=m_adMax[m_iCurrent] )
				break;
	return m_adMultiplier[m_iCurrent];
	}
	
//-----------------------------------------------------------------------------		
bool CGainTable :: LoadFromFile( FILE * h, int iVer ){ 
	m_bNew=false;
	//-- Read Table name 
	if( ! FileLoadString( h, &m_cTableName ) )
		return false;
	//-- Read active Entries count
	if( fread( &m_iEntries, sizeof(int), 1, h ) != 1 ) 
		return false;
	//-- Read Signals and multipliers
	if( fread( m_adSignal, sizeof(double), GAIN_MAXENTRIES, h ) != GAIN_MAXENTRIES ) 
		return false;
	if( fread( m_adMultiplier, sizeof(double), GAIN_MAXENTRIES, h ) != GAIN_MAXENTRIES ) 
		return false;
	//-- Calculate remaining values 
	m_iCurrent=0;
	SetRanges();
	return true;
	}
	
//-----------------------------------------------------------------------------		
bool CGainTable :: SaveToFile( FILE * h ) { 
	m_bNew=false;
	//-- Read Table name 
	if( ! FileSaveString( h, m_cTableName.ConstBuffer() ) )
		return false;
	//-- Read active Entries count
	if( fwrite( &m_iEntries, sizeof(int), 1, h ) != 1 ) 
		return false;
	//-- Read Signals and multipliers
	if( fwrite( m_adSignal, sizeof(double), GAIN_MAXENTRIES, h ) != GAIN_MAXENTRIES ) 
		return false;
	if( fwrite( m_adMultiplier, sizeof(double), GAIN_MAXENTRIES, h ) != GAIN_MAXENTRIES ) 
		return false;
	return true;
	}

int CGainTable :: SaveSize() {
	return m_cTableName.Length()+5
		+ sizeof(m_iEntries)
		+ sizeof(double)*GAIN_MAXENTRIES		// m_adSignal
		+ sizeof(double)*GAIN_MAXENTRIES;		// m_adMultiplier
	}

//=============================================================================
CChannelConfig :: CChannelConfig() { 
	Clear();
	}
	
 void CChannelConfig :: Clear() { 
	m_iActive=0;
	m_cChannelName.Set("");
	m_dGain=1.0;
	m_cUnits.Set("");
	m_dScaling=1.0;
	}
				
//-----------------------------------------------------------------------------		
bool CChannelConfig :: LoadFromFile( FILE * h ){ 
	//-- Read Active flag 
	if( fread( &m_iActive, sizeof(int), 1, h ) != 1 ) 
		return false;
	//-- Read channel name 
	if( ! FileLoadString( h, &m_cChannelName ) )
		return false;
	//-- Read Gain Setting ( ignored for AO ) 
	if( fread( &m_dGain, sizeof(double), 1, h ) != 1 ) 
		return false;
	//-- Read Units 
	if( ! FileLoadString( h, &m_cUnits ) )
		return false;
	//-- Read Scaling factor
	if( fread( &m_dScaling, sizeof(double), 1, h ) != 1 ) 
		return false;
	return true;
	}
	
//-----------------------------------------------------------------------------		
bool CChannelConfig :: SaveToFile( FILE * h ) { 
	//-- Write Active flag 
	if( fwrite( &m_iActive, sizeof(int), 1, h ) != 1 ) 
		return false;
	//-- write channel name 
	if ( ! FileSaveString( h, m_cChannelName.Buffer() ) )
		return false;
	//-- write Gain Setting ( ignored for AO ) 
	if( fwrite( &m_dGain, sizeof(double), 1, h ) != 1 ) 
		return false;
	//-- write Units 
	if( ! FileSaveString( h,  m_cUnits.Buffer() ) )
		return false;
	//-- write Scaling factor
	if( fwrite( &m_dScaling, sizeof(double), 1, h ) != 1 ) 
		return false;
	return true;
	}

int CChannelConfig :: SaveSize() {
	return sizeof(m_iActive)
		+ m_cChannelName.Length()+5
		+ sizeof(m_dGain)
		+ m_cUnits.Length()+5
		+ sizeof(m_dScaling);
	}

//=============================================================================
CConfiguration :: CConfiguration() { 
	m_iAcqMeasMode=0;
	m_cConfigName.Set("");
	m_cDeviceName.Set("");
	char buf[10];
	for( int i=0; i<CONFIG_DOROWS; ++i ) {
		sprintf( buf, "DigOut%2d", i );
		m_acDigName[i].Set( buf );
		m_aiDigMode[i]=DOMODE_MANUAL;
		}
	}	

//-----------------------------------------------------------------------------		
bool CConfiguration :: LoadFromFile( FILE * h, int iVer ){ 
	int i, iAI,iAO;

	//-- Config and Device names
	if( ! FileLoadString( h, &m_cConfigName ) 
		|| ! FileLoadString( h, &m_cDeviceName ) ) 
		return false;
	//-- Array of input channels
	if( fread( &iAI, sizeof(int), 1, h ) != 1 ) 
		return false;
	for( i=0; i<iAI; ++i )
		if( ! m_aoInChnCfg[i].LoadFromFile(h) ) 
			return false;
	for( i=iAI; i<CONFIG_AIROWS; ++i ) 
		m_aoInChnCfg[i].Clear();
	//-- Array of output channels
	if( fread( &iAO, sizeof(int), 1, h ) != 1 ) 
		return false;
	for( i=0; i<iAO; ++i )
		if( ! m_aoOutChnCfg[i].LoadFromFile(h) ) 
			return false;
	for( i=iAO; i<CONFIG_AOROWS; ++i ) 
		m_aoOutChnCfg[i].Clear();
		
	//-- lpt1 Names - removed, skip over lpt1 names for prior version compatibility
	if( iVer>=2 && iVer<7 ) {
		CString cJunk;
		for( i=0; i<8; ++i ) 
			if( ! FileLoadString( h, &cJunk ) )
				return false;
		}

	//-- Digital output  Names 
	char buf[10];
	if( iVer>=6 ) {
		for( i=0; i<CONFIG_DOROWS; ++i )
			if( ! FileLoadString( h, &(m_acDigName[i]) ) )
				return false;
		if( fread( &m_aiDigMode, CONFIG_DOROWS*sizeof(m_aiDigMode[0]), 1, h ) != 1 ) 
			return false;
		}
	else if( iVer>=4 ) {
		for( i=0; i<8; ++i ) 
			if( ! FileLoadString( h, &(m_acDigName[i]) ) )
				return false;
		for( i=8; i<CONFIG_DOROWS; ++i ) {
			sprintf( buf, "DigOut%2d", i );
			m_acDigName[i].Set( buf );
			}
		if( fread( &m_aiDigMode[0], sizeof(m_aiDigMode[0]), 1, h ) != 1 ) 
			return false;
		for( i=1; i<CONFIG_DOROWS; ++i ) 
			m_aiDigMode[i]=m_aiDigMode[0];
		}
	else {
		for( i=0; i<CONFIG_DOROWS; ++i ) {
			sprintf( buf, "DigOut%2d", i );
			m_acDigName[i].Set( buf );
			}
		for( i=0; i<CONFIG_DOROWS; ++i ) 
			m_aiDigMode[i]=1;
		}

	//-- measurement mode - RSE / NRSE / Diff
	if( iVer>=8 ) { 
		BYTE iMode;
		if( fread( &iMode, 1, 1, h ) != 1 ) 
			return false;
		m_iAcqMeasMode=iMode;
		}
	
	return true;
	}
	
//-----------------------------------------------------------------------------		
bool CConfiguration :: SaveToFile( FILE * h ) { 
	int i, iAI=CONFIG_AIROWS,iAO=CONFIG_AOROWS;
	//-- Config and Device names
	if( ! FileSaveString( h, m_cConfigName.Buffer() ) 
		|| ! FileSaveString( h, m_cDeviceName.Buffer() ) ) 
		return false;
	//-- Array of input channels
	if( fwrite( &iAI, sizeof(int), 1, h ) != 1 ) 
		return false;
	for( i=0; i<iAI; ++i )
		if( ! m_aoInChnCfg[i].SaveToFile(h) ) 
			return false;
	//-- Array of output channels
	if( fwrite( &iAO, sizeof(int), 1, h ) != 1 ) 
		return false;
	for( i=0; i<iAO; ++i )
		if( ! m_aoOutChnCfg[i].SaveToFile(h) ) 
			return false;
	//-- Digital output  Names 
	for( i=0; i<CONFIG_DOROWS; ++i ) 
		if( ! FileSaveString( h, m_acDigName[i].Buffer() ) )
			return false;
	//-- Digital auto reset 
	if( fwrite( &m_aiDigMode, sizeof(m_aiDigMode[0])*CONFIG_DOROWS, 1, h ) != 1 ) 
		return false;
	//-- measurement mode - RSE / NRSE / Diff
	BYTE iMode=m_iAcqMeasMode;
	if( fwrite( &iMode, 1, 1, h ) != 1 ) 
		return false;

	return true;
	}

//-----------------------------------------------------------------------------		
int CConfiguration :: SaveSize() {
	int i, iAISize=sizeof(int);
	for( i=0; i<CONFIG_AIROWS; ++i )
		iAISize += m_aoInChnCfg[i].SaveSize();

	int iAOSize=sizeof(int);
	for( i=0; i<CONFIG_AOROWS; ++i )
		iAOSize += m_aoOutChnCfg[i].SaveSize();

	int iDigoutSize=0;
	for( i=0; i<CONFIG_DOROWS; ++i ) 
		iDigoutSize += m_acDigName[i].Length()+5;

	return m_cConfigName.Length()+5
		+ m_cDeviceName.Length()+5
		+ iAISize + iAOSize + iDigoutSize
		+ sizeof(m_aiDigMode[0])*CONFIG_DOROWS
		+ 1;		//Mode 
	}

//-----------------------------------------------------------------------------		
int CConfiguration :: ActiveAcqChannels( ) { 
	int iCount=0;
	for( int iCh=0; iCh<CONFIG_AIROWS; ++iCh ) 
		if( m_aoInChnCfg[iCh].m_iActive != 0 )
			++iCount;
	return iCount;
	} 
	
int CConfiguration :: ActiveGenChannels( ) { 
	int iCount=0;
	for( int iCh=0; iCh<CONFIG_AOROWS; ++iCh ) 
		if( m_aoOutChnCfg[iCh].m_iActive != 0 )
			++iCount;
	return iCount;
	} 

//-----------------------------------------------------------------------------		
bool CConfiguration :: HasProtocolDigGen() { 
	for( int iLine=0; iLine<MAXDOLINES; ++iLine ) 
		if( m_aiDigMode[iLine]==DOMODE_PROTOCOL ) 
			return true;
	return false;
	}
	
int CConfiguration :: DigGenProtocolCount( BYTE ** paiDigGenMap ) {
	int iLines=0;
	for( int iLine=0; iLine<MAXDOLINES; ++iLine ) 
		if( m_aiDigMode[iLine]==DOMODE_PROTOCOL ) 
			++iLines;
	
	if( paiDigGenMap!=NULL ) { 
		if( *paiDigGenMap != NULL ) 
			delete [] *paiDigGenMap;
		*paiDigGenMap = new BYTE[iLines];
		iLines=0;
		for( int iLine=0; iLine<MAXDOLINES; ++iLine ) 
			if( m_aiDigMode[iLine]==DOMODE_PROTOCOL ) 
				(*paiDigGenMap)[iLines++]=iLine;
		}
		
	return iLines;
	}

int CConfiguration :: DigAcqCount( BYTE ** paiDigAcqMap ) {
	int iLines=0;
	for( int iLine=0; iLine<MAXDOLINES; ++iLine ) 
		if( m_aiDigMode[iLine]==DOMODE_INPUT ) 
			++iLines;
	
	if( paiDigAcqMap!=NULL ) { 
		if( *paiDigAcqMap != NULL ) 
			delete [] *paiDigAcqMap;
		*paiDigAcqMap = new BYTE[iLines];
		iLines=0;
		for( int iLine=0; iLine<MAXDOLINES; ++iLine ) 
			if( m_aiDigMode[iLine]==DOMODE_INPUT ) 
				(*paiDigAcqMap)[iLines++]=iLine;
		}
		
	return iLines;
	}



//=============================================================================
bool CConfigurations :: LoadFile( ) { 

	//-- Locate file
	if ( ! g_ConfigLoc ) {
		g_ConfigLoc = (char *) malloc(512);
		getcwd(g_ConfigLoc, 512);
		int p = strlen(g_ConfigLoc);
		sprintf(g_ConfigLoc+p, "\\%s", CONFIGFILENAME);
	}
	//-- Open File
	FILE * h = fopen( g_ConfigLoc, "rb" );
	if( h == NULL ) 
		return false;
	
	//-- Get Version Number, Hardware pos and config count.
	int iVer, iCfgPos, iConfigs;
	char buf[4];
	if( fread( buf, 4, 1, h ) != 1 ) {
		fclose(h);
		return false;
		}
	if( memcmp( buf, "QCFG", 4) == 0 ) { 
		iVer=0;
		if( fread( &iCfgPos, sizeof(int), 1, h ) != 1
			|| fseek( h, iCfgPos, SEEK_SET ) != 0 
			|| fread( &iConfigs, sizeof(iConfigs), 1, h ) != 1 ) { 
			fclose(h);
			return false;
			}
		}
	else if( memcmp( buf, "QCNF", 4) == 0 ) { 
		if( fread( &iVer, sizeof(int), 1, h ) != 1 ) {
			fclose(h);
			return false;
			}
		if( fread( &iCfgPos, sizeof(int), 1, h ) != 1
			|| fread( &iConfigs, sizeof(iConfigs), 1, h ) != 1 ) { 
			fclose(h);
			return false;
			}
		}
	else {
		fclose(h);
		return false;
		}

	//-- Get Settings pos 
	int iSetPos = ftell(h);
	if( iVer>=1 && fread( &iSetPos, sizeof(int), 1, h ) != 1 ){
		fclose(h);
		return false;
		}
		
	//-- Get Gain Tables pos and count
	int iGainTableCount, iGainTablePos;
	if( iVer>=3 ) {
		//-- Get Gain Table Pos and count		
		if( fread( &iGainTablePos, sizeof(int), 1, h ) != 1 
				|| fread( &iGainTableCount, sizeof(int), 1, h ) != 1 ) {
			fclose(h);
			return false;
			}
		}

	//-- Get Feedback pos and count
	int iFeedbackCount, iFeedbackPos;
	if( iVer>=5 ) {
		//-- Get Feedback Config Pos and count		
		if( fread( &iFeedbackPos, sizeof(int), 1, h ) != 1 
					|| fread( &iFeedbackCount, sizeof(int), 1, h ) != 1 ) {
			fclose(h);
			return false;
			}
		}

	//-- Get Gain Tables 
	m_aGainTables.DeleteAll();
	if( iVer>=3 ) {
		fseek( h, iGainTablePos, SEEK_SET );
		CGainTable * pGainTable;
		for( int i=0; i<iGainTableCount; ++i ) {
			pGainTable = new CGainTable;
			if( pGainTable == NULL || ! pGainTable->LoadFromFile(h, iVer) ) {
				fclose(h);
				return false;
				}
			m_aGainTables.AddItem(pGainTable);
			}
		}
	
	//-- Get Feedback array
	m_aFeedbackConfigs.DeleteAll();
	if( iVer>=5 ) {
		fseek( h, iFeedbackPos, SEEK_SET );
		CFeedback * pFeedback;
		for( int i=0; i<iFeedbackCount; ++i ) {
			if( ! CFeedback::LoadFromFile( &pFeedback, h, iVer ) ) {
				fclose(h);
				return false;
				}
			m_aFeedbackConfigs.AddItem(pFeedback);
			}
		}

	//-- Get Settings
	if( iVer>=1 ) { 
		if( fseek( h, iSetPos, SEEK_SET ) != 0 
			|| ! m_oSettings.LoadFromFile( h ) ) {
			fclose(h);
			return false;
			}
		}
	
	//-- Get Configs Array
	if( iVer>=1 )
		fseek( h, iCfgPos, SEEK_SET );
	CConfiguration * pCfg;
	m_aConfigs.DeleteAll();
	for( int i=0; i<iConfigs; ++i ) {
		pCfg = new CConfiguration;
		if( pCfg == NULL || ! pCfg->LoadFromFile(h, iVer) ) {
			fclose(h);
			return false;
			}
		m_aConfigs.AddItem(pCfg);
		}
			
	//-- Close file
	fclose(h);
	return true;
	}
	
//-----------------------------------------------------------------------------		
bool CConfigurations :: SaveFile( ) { 
	//-- Open File
	FILE * h = fopen( CONFIGFILETEMP, "wb" );
	if( h == NULL ) 
		return false;
	
	//-- Write file header 
	int iVer=8;
	int iSetPos=36;			
	int iCfgPos=iSetPos+m_oSettings.SaveSize();
	int iGainPos=iCfgPos+ConfigsSize();
	int iFeedbackPos=iGainPos+GainTableSize();
	//int i...<whatever comes next>...=iFeedbackPos+FeedbackConfigSize(), also change iSetPos!
	int iCfgCount=m_aConfigs.Count();
	int iGainCount=m_aGainTables.Count();
	int iFeedbackCount=m_aFeedbackConfigs.Count();
	if( fwrite( "QCNF", 4, 1, h ) != 1					//--  0 Write file signature 
		|| fwrite( &iVer, sizeof(int), 1, h ) != 1		//--  4 write version number 
		|| fwrite( &iCfgPos, sizeof(int), 1, h ) != 1	//--  8 write address of hardware configs.
		|| fwrite( &iCfgCount, sizeof(int), 1, h ) != 1	//-- 12 Write Config Count
		|| fwrite( &iSetPos, sizeof(int), 1, h ) != 1	//-- 16 address of settings
		|| fwrite( &iGainPos, sizeof(int), 1, h ) != 1	//-- 20 address of gain table array  
		|| fwrite( &iGainCount, sizeof(int), 1, h )!= 1	//-- 24 items in gain table array 
		|| fwrite( &iFeedbackPos, sizeof(int), 1, h ) != 1	//-- 28 address of gain table array  
		|| fwrite( &iFeedbackCount, sizeof(int), 1, h )!= 1	//-- 32 items in gain table array 
		|| ! m_oSettings.SaveToFile( h )				//-- Settings contents
		) {	
		fclose(h);
		return false;
		}
		
	//-- Write Configs Array
	for( int i=0; i<iCfgCount; ++i ) 
		if( ! Config(i)->SaveToFile(h) ) {
			fclose(h);
			return false;
			}
			
	//-- Write Gain Table Array
	for( int i=0; i<iGainCount; ++i ) 
		if( ! GainTable(i)->SaveToFile(h) ) {
			fclose(h);
			return false;
			}
			
	//-- Write Feedback array
	for( int i=0; i<iFeedbackCount; ++i ) {
		if( ! FeedbackConfig(i)->SaveToFile(h) ) { 
			fclose(h);
			return false;
			}
		}

	//-- Close file 
	fclose(h);
	
	//-- file written successfully, rename files 
	DeleteFile( CONFIGFILEBACK );
	MoveFile( g_ConfigLoc, CONFIGFILEBACK );
	return MoveFile( CONFIGFILETEMP, g_ConfigLoc ) ? true : false;
	}
		
bool CConfigurations :: ConfigFileExists( )  { 
	return FileExists( CONFIGFILENAME );
	}

//-----------------------------------------------------------------------------		
int CConfigurations :: GainTableSize() {
	int iSize=0;
	for( int i=0; i<m_aGainTables.Count(); ++i ) 
		iSize += GainTable(i)->SaveSize();
	return iSize;	
	}

int CConfigurations :: FeedbackConfigSize() {
	int iSize=0;
	for( int i=0; i<m_aFeedbackConfigs.Count(); ++i ) 
		iSize += sizeof(int)+FeedbackConfig(i)->SaveSize();	// added int for base class -> Feedback type being written.
	return iSize;	
	}

int CConfigurations :: ConfigsSize() {
	int iSize=0;
	for( int i=0; i<m_aConfigs.Count(); ++i ) 
		iSize += Config(i)->SaveSize();
	return iSize;
	}
		
//-----------------------------------------------------------------------------		
int CConfigurations :: FindConfig( const char * pchConfigName ) { 
	for( int iCfg=0; iCfg<m_aConfigs.Count(); ++iCfg ) 
		if( strcmp( ((CConfiguration *)m_aConfigs[iCfg])->m_cConfigName.Buffer(), pchConfigName) == 0 ) 
			return iCfg;
	return -1;
	}

//-----------------------------------------------------------------------------		
bool CConfigurations :: AddConfig( CConfiguration * poConfig ) { 
	m_aConfigs.AddItem( poConfig );
	return true;
	}
	
//-----------------------------------------------------------------------------		
bool CConfigurations :: DeleteConfig( const char * pchConfigName ) {
	int i = FindConfig( pchConfigName );
	if( i == -1  )
		return false;
	m_aConfigs.DeleteItem( i );
	return true;
	}

//-----------------------------------------------------------------------------		
bool CConfigurations :: DeleteConfig( int iConfig ) {
	if( iConfig<0 || iConfig>=m_aConfigs.Count() )
		return false;
	m_aConfigs.DeleteItem( iConfig );
	return true;
	}

//-----------------------------------------------------------------------------		
CConfiguration * CConfigurations :: Config( int i ) { 
	if( i<0 || i>=m_aConfigs.Count()  )
		return NULL;
	return (CConfiguration *)m_aConfigs[i];
	}

//-----------------------------------------------------------------------------		
CGainTable * CConfigurations :: GainTable( int i ) { 
	if( i<0 || i>=m_aGainTables.Count()  )
		return NULL;
	return (CGainTable *) m_aGainTables[i];
	}
	
CGainTable * CConfigurations :: GainTable( const char * pchTableName ) {
	for( int i=0; i<m_aGainTables.Count(); ++i ) 
		if( ((CGainTable *) m_aGainTables[i])->m_cTableName==pchTableName ) 
			return (CGainTable *) m_aGainTables[i];
	return NULL;
	}


//-----------------------------------------------------------------------------		
CFeedback * CConfigurations :: FeedbackConfig( int i ) {
	if( i<0 || i>=m_aFeedbackConfigs.Count()  )
		return NULL;
	return (CFeedback *) m_aFeedbackConfigs[i];
	}
	
CFeedback * CConfigurations :: FeedbackConfig( const char * pchFeedbackConfigName ) {
	for( int i=0; i<m_aFeedbackConfigs.Count(); ++i ) 
		if( ((CFeedback *) m_aFeedbackConfigs[i])->m_cName==pchFeedbackConfigName ) 
			return (CFeedback *) m_aFeedbackConfigs[i];
	return NULL;
	}
	
	
	
	
